<?php

namespace Controller;

use Service\Traits\TResponse;
use Service\ColumnsQueryService;
use Service\ConnectionService;
use Service\DataSourceService;
use Service\DeleteCascadeService;
use Zeedhi\Framework\DataSource\Manager;
use Util\Utilities;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO;
class StructureType extends Crud{
    
    use TResponse;

    const ERRORS_CONNECT_FAIL = "PHP Error occurred 'oci_connect()";
    const ERRORS_EXECUTE_QUERY_FAIL = "PHP Error occurred 'oci_execute()";

    const RESPONSE_STRING_CONNECTION_NOT_FOUND = "Connection not found: ";
    const RESPONSE_STRING_FAIL_AT_CREATE = "Wasn't possible create the connection: ";
    const RESPONSE_STRING_CONNECTION_FAIL = "Wasn't possible connect with database, the old columns were kept.";
    const RESPONSE_STRING_EXECUTE_QUERY_FAIL = "Wasn't possible sync your columns. Fail to execute query: ";

    protected $dataSourceName = 'structureType';

    /** @var Connection */
    protected $connection;

    protected $dataSourceService;

    protected $dataSourceOtherService;

    protected $socketClient;

    protected $environment;

    /** @var ConnectionService */
    protected $connectionService;

    /** @var StructureManagerialService */
    protected $structService;

    public function __construct(Manager $organizationMongoManager, DeleteCascadeService $deleteCascadeService, ConnectionService $connectionService, \Service\DataSourceService $dataSourceService, \Service\DataSource $dataSourceOtherService, \Service\SocketClient $socketClient, \Util\Environment $environment, \Service\StructureManagerialService $structService)
    {   
        parent::__construct($organizationMongoManager, $deleteCascadeService);
        $this->connectionService = $connectionService;
        $this->socketClient = $socketClient;
        $this->dataSourceService = $dataSourceService;
        $this->dataSourceOtherService = $dataSourceOtherService;
        $this->environment = $environment;
        $this->structService = $structService;
        $this->mongoManager->createDatabase(\Util\Utilities::getOrganizationIdForMongo());
    }

    public function findNext1(DTO\Request $request, DTO\Response $response) {
        $params = $request->getParameters();
        //	parent::find($request, $response);
        $structureTypes = $this->mongoManager->findNext($this->dataSourceName, $params);
        $connections = $this->getConnections($structureTypes)->getRows();
        $indexedConnections = $this->indexByMongoId($connections);
        $hierarchyTypes = $this->relateStructureTypeWithConnection($structureTypes, $indexedConnections);
        
        $data = Utilities::parseDataSourceZeedhiNext($hierarchyTypes);
        $data["pagination"]["total"] = $this->count($params);
        $data["pagination"]["page"] = isset($params["page"]) ? (int)$params["page"] : 1;
        $data["pagination"]["limit"] = isset($params["limit"]) ? (int)$params["limit"] : 9999;
        
        $this->sendResponse([
	        "data" => $data,
	        "pagination" => ["total" => $total, "page" => $page, "limit" => $limit]
        ], 200);
        //  $newDataSet = new DataSet($this->dataSourceName, $hierarchyTypes);
        //  $response->setDataSets(array($newDataSet));
    }

    public function count($params) {
        return $this->mongoManager->countNext($this->dataSourceName, $params);
    }

    public function findNext(DTO\Request $request, DTO\Response $response)
    {
        $params = $request->getParameters();
        //$structTypes = $this->structService->getStructType($params);
        //$connections = $this->getConnectionsNext($structTypes);
        $dataSet = $this->structService->findNext($params);
        // $indexedConnections = $this->indexByMongoId($connections);
        // $hierarchyTypes = $this->relateStructureTypeWithConnection($structureTypes, $indexedConnections);
        // $newDataSet = new DataSet($structTypes);
        // $response->addDataSet($newDataSet);
        $data = Utilities::parseDataSourceZeedhiNext($dataSet, $this->count($params));
        $data["pagination"]["page"] = isset($params["page"]) ? (int)$params["page"] : 1;
        $data["pagination"]["limit"] = isset($params["limit"]) ? (int)$params["limit"] : 99999;
        $this->sendResponse($data, 200);
    }

    protected function getConnectionsNext($structureTypes)
    {
        $usedIds = $this->getConnectionsIdsUsed($structureTypes);
        $numberOfExpectedLines = count($usedIds);

        return $this->structService->getConnection($usedIds, $numberOfExpectedLines);
    }


    protected function getConnections($structureTypes)
    {
        $usedIds = $this->getConnectionsIdsUsed($structureTypes);
        $numberOfExpectedLines = count($usedIds);
        $filterCriteria = new FilterCriteria('connection', array(), null, $numberOfExpectedLines);
        $filterCriteria->addCondition('_id', FilterCriteria::IN, $usedIds);
        return $this->mongoManager->findBy($filterCriteria);
    }

    protected function getConnectionsIdsUsed($structureTypes)
    {
        //For not repeat Ids used more than one time
        $connectionIdsIndexedById = $this->getDistinctConnectionsIds($structureTypes);
        //Build distinct ids in a index array
        $distictConnectionIds = array();
        foreach ($connectionIdsIndexedById as $connectionId) {
            $distictConnectionIds[] = $connectionId;
        }
        return $distictConnectionIds;
    }

    protected function getDistinctConnectionsIds($structureTypes)
    {
        $connectionsIdsIndexedById = array();
        foreach ($structureTypes as $structureType) {
            if (isset($structureType['connection'])) {
                $connectionId = $structureType['connection'];
                $connectionsIdsIndexedById[$connectionId] = $connectionId;
            }
        }
        return $connectionsIdsIndexedById;
    }

    protected function indexByMongoId($rows)
    {
        $newRows = array();
        foreach ($rows as $row) {
            $newRows[$row['_id']] = $row;
        }
        return $newRows;
    }

    protected function relateStructureTypeWithConnection($structureTypes, $indexedByIdConnections)
    {
        foreach ($structureTypes as &$structureType) {
            if (isset($structureType['connection']) && isset($indexedByIdConnections[$structureType['connection']])) {
                $structureType['connectionName'] = $indexedByIdConnections[$structureType['connection']]['name'];
            }
        }
        return $structureTypes;
    }

    public function getFieldKeysFromQuery(DTO\Request $request, DTO\Response $response)
    {   $params = $request->getParameters();
        try {
            //TODO: use custom connection if there is one (first it need to be implemented in the websocket server)
            $columns = $this->getColumnsFromWebSocket($params);
            $this->sendResponse($columns, 200); 
        } catch (Exception $e) {
            if ($e->getCode() === Exception::CODE_FAIL_CONNECTION || $e->getCode() === Exception::CODE_QUERY_EXECUTION_FAIL) {
                $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
            } else {
                throw $e;
            }
        }
    }

    protected function getColumnsFromWebSocket($row)    {   
        try {
            if (isset($row['add'])) {
                $cleanedQuery = $this->dataSourceOtherService->replaceQueryParameters($row['queryAdd']);    
            }

            if (isset($row['edit'])) {
                $cleanedQuery = $this->dataSourceOtherService->replaceQueryParameters($row['query']);
            }
            $organizationID = Utilities::getOrganizationIdForMongo();
            return $this->dataSourceService->getMergedColumnsFromWebSocket($this->socketClient, $cleanedQuery, $row, $organizationID);
        } catch (\Zeedhi\Framework\ErrorHandler\ErrorException $e) {

            if (strrpos($e->getMessage(), self::ERRORS_CONNECT_FAIL)) {
                throw Exception::connectFail();
            } else if (strrpos($e->getMessage(), self::ERRORS_EXECUTE_QUERY_FAIL)) {
                throw Exception::executeQueryError($row['query'], $e);
            } else {
                throw $e;
            }
        }
    }
}
