<?php

namespace Service\ChartView;

use Zeedhi\Framework\DependencyInjection\InstanceManager;
class Column extends ChartViewAbstract implements ChartViewInterface {

    const TYPE_ID_COLUMN = "579106a111e8f41bb00caff6";

    public $widget;
    public $widgetData;

    function __construct($widget = null) {
        $this->widget = $widget;
        $this->widgetData = [];
    }

    protected function getXAxis(){
        $widget = $this->widget;
        $data = $this->widgetData;
        $categoria = [];
        foreach ($data as $row) {
            if(!empty($row[$widget['chartXAxisField']]) && !in_array($row[$widget['chartXAxisField']], $categoria)){
                $categoria[] = $row[$widget['chartXAxisField']];
            }
        }
        $xAxis = [
            "mask" => !empty($widget['chartXAxisFormat']) ? $widget['chartXAxisFormat'] : null,
            "categories" => $categoria,
            "field" => $widget['chartXAxisField'],
            "labels" => [
                "show" => true,
                "rotate" => !empty($widget['chartXAxisTicksRotation']) ? $widget['chartXAxisTicksRotation']-90 : -60,
                "trim" => true
            ],
            "tickPlacement" => "between"
        ];

        if($xAxis["labels"]["rotate"] != 0){
            $xAxis["labels"]["rotateAlways"] = false;
        }

        return $xAxis;
    }

    protected function getYAxis(){
        $widget = $this->widget;
        $yAxis = [
            "mask" => $widget['chartSeries'][0]['serieYAxisFormat'],
            "dataLabels" => [],
            "labels" => [
                "show" => true,
            ],
            "opposite" => !empty($widget["serieYAxisPosition"]) ? $widget["serieYAxisPosition"] == "right" : false,
        ];

        if(!empty($widget['serieYAxisLabel'])){
            $yAxis['title'] = [
                "text" => $widget['serieYAxisLabel'],
                "style" => [
                    "fontSize" => "16px"
                ]
            ];
        }

        return $yAxis;
    }

    protected function getSeries(){
        $widget = $this->widget;
        $series = [];

        $serieMap = [];
        foreach ($this->widgetData as $row) {
            if (!empty($row[$widget["chartXAxisField"]])){
                $dimension = $row[$widget["chartXAxisField"]];
                if( empty($serieMap[$dimension]) ){
                    $serieMap[$dimension] = [];
                }
                foreach ($widget['serieFields'] as $fact) {
                    if(empty($serieMap[$dimension][$fact])){
                        $serieMap[$dimension][$fact] = 0;
                    }
                    $serieMap[$dimension][$fact] += $row[$fact];
                }
            }
        }

        foreach ($this->widgetData as $row) {
            foreach ($widget['serieFields'] as $index => $fact) {
                if(empty($series[$index])){
                    $data = array_column($serieMap, $fact);
                    $series[$index] = array(
                        "name" => $widget['serieLabels'][$index] ,
                        "field" => $fact,
                        "data" => $data,
                    );
                }
            }
        }
        return $series;
    }

    protected function getTooltip(){
        $widget = $this->widget;
        return [
            "enabled" => !empty($widget["chartShowTooltip"]) ? $widget["chartShowTooltip"] : false
        ];
    }

    protected function getPlotOptions(){
        $widget = $this->widget;
        $barOptions = [
            "dataLabels" => [
                "position" => !empty($widget['chartInvertedAxes']) ? 'left' : 'top',
                "textAnchor" => "middle",
            ],
        ];

        return [
            "bar" => $barOptions
        ];
    }

    protected function getSerieStacked(){
        $widget = $this->widget;

        if(!empty($widget['serieStacked'])){
            return $widget['serieStacked'];
        }
        return false;
    }

    protected function getChart(){
        $widget = $this->widget;

        return [
            "type" => "bar",
            "id" => $widget['id'],
            "stacked" => $this->getSerieStacked(),
            "toolbar" => [
                "show" => true,
                "tools" => [
                    "download" => true,
                    "selection" => false,
                    "zoom" => false,
                    "zoomin" => false,
                    "zoomout" => false,
                    "pan" => false,
                    "reset" => false,
                    "customIcons" => []
                ],
            ],
        ];
    }

    protected function getSerieColors(){
        $widget = $this->widget;
        if(!empty($widget["chartSeries"][0]["serieColors"])){
            return $widget["chartSeries"][0]["serieColors"];
        }
        return self::DEFAULT_COLORS;
    }

    protected function getDataLabels(){
        $widget = $this->widget;
        return [
            "enabled" => !empty($widget["serieShowValueLabels"]) ? $widget["serieShowValueLabels"] : false,
            "position" => "top",
            "textAnchor" => "middle",
            "offsetY" => -20,
            "style" => [
                "colors" => $this->getSerieStacked() ? [] : $this->getSerieColors()
            ],
        ];
    }

    protected function getTitle(){
        $widget = $this->widget;
        return [
            "text" => $widget['name'],
            "style" => [
                "fontSize" => "16px"
            ]
        ];
    }

    protected function getNoData(){
        $widget = $this->widget;

        return [
            "text" => !empty($widget["chartMessageNoData"]) ? $widget["chartMessageNoData"] : self::NO_DATA_TEXT,
            "align" => 'center',
            "verticalAlign" => 'middle',
            "offsetX" => 0,
            "offsetY" => 0,
            "style" => [
              "fontSize" => '14px',
            ],
        ];
    }

    protected function getOptions(){
        $widget = $this->widget;

        $legendPosition = "top";
        if($widget["chartLegendPosition"]){
            $legendPosition = explode(" ",$widget["chartLegendPosition"])[0];
        }

        return [
            "colors" => $this->getSerieColors(),
            "drillDownView" => empty($widget['drillDownView']) ? null : $widget['drillDownView'],
            "plotOptions" => $this->getPlotOptions(),
            "dataLabels" => $this->getDataLabels(),
            "chart" => $this->getChart(),
            "legend" => [
                "position" => $legendPosition
            ],
            "zoom" > [
                "enabled" => true
            ],
            "title" => $this->getTitle(),
            "noData" => $this->getNoData(),
            "tooltip" => $this->getTooltip(),
            "xaxis" => $this->getXAxis(),
            "yaxis" => $this->getYAxis()
        ];
    }

    public function convertToNext():array {
        $this->fetchWidgetData();
        $widget = $this->widget;

        $indicatorNext = [
            "name" => "apexChart_" . $widget['id'],
            "id" => $widget['id'],
            "widget" => $widget,
            "viewTypeId" => self::TYPE_ID_COLUMN,
            "component" => self::CHART_COMPONENT,
            "chartType" => $widget['serieType'],
            "height" =>"100%",
            "fillHeight" => true,
            "cssStyle" => self::CHART_CSS_STYLE,
            "options" => $this->getOptions(),
            "series" => $this->getSeries()
        ];

        $this->widget = $indicatorNext;
        return $indicatorNext;
    }
}
