<?php

namespace Service\ChartView;

use Zeedhi\Framework\DependencyInjection\InstanceManager;
class Gauge extends ChartViewAbstract implements ChartViewInterface {

    const TYPE_ID_GAUGE = "57ab6a47574698ae9522f97e";

    public $widget;
    public $widgetData;

    function __construct($widget = null) {
        $this->widget = $widget;
        $this->widgetData = [];
        $this->sortedRangeValues = [];
    }

    protected function getSeries(){
        $widget = $this->widget;
        $widgetData = $this->widgetData;
        $stringPointerL = $widget['serieGaugeWidth'] * 100;
        $colorPointer = array_key_exists('seriePointerColor', $widget) ? $widget['seriePointerColor'] : '';
        return  [
            [
              'type' => 'gauge',
              'radius' => "100%",
              "center" => ["50%","75%"],
              'startAngle' => 180,
              'endAngle' => 0,
              "max" => $this->maxValue($widget, $widgetData),
              "min" => reset($this->sortedRangeValues),
              'axisLine' => [
                'lineStyle' => [
                  'width' => 30,
                  "color" => $this->getColors(),
                ],
              ],
              'axisTick' => [
                'show' => false,
              ],
              'pointer' => [
                'length' => strval($stringPointerL).'%',
                'itemStyle' => [
                  'color' => $colorPointer,
                ],
              ],
              'axisLabel' => [
                'color' => 'inherit',
                'distance' => 40,
                'fontSize' => 20,
                'formatter' => null,
              ],
              'detail' => [
                'valueAnimation' => true,
                'formatter' => null,
                'color' => 'inherit',
                "offsetCenter" => [0, "10%"],
              ],
              'data' => [
                [
                  'value' => $this->widgetData[0][$widget["serieValue"]],
                ],
              ],
            ],
        ];
    }
    

    protected function sortRangeValues(){
        $orderedValues = [];
        foreach($this->widget["zmartSerieRanges"] as $value){
            $orderedValues[$value] =  $this->widgetData[0][$value];
        }
        asort($orderedValues);
        $this->sortedRangeValues = $orderedValues;
    }

    protected function maxValue($widget, $widgetData) {
      if (array_key_exists('zmartSerieMaxValue', $widget) === true) {
          return $this->widgetData[0][$widget["zmartSerieMaxValue"]];
      }
      return end($this->sortedRangeValues);
    }

    protected function getColors(){
        $widget = $this->widget;
        $widgetData = $this->widgetData;
        $maxValue = end($this->sortedRangeValues);
        $minValue = reset($this->sortedRangeValues);
        $colors = [];
        if (array_key_exists('serieColors', $widget) === true) {
          $serieColors = $widget["serieColors"];
          $i = 0;
          foreach($this->sortedRangeValues as $value){
              if($value != $minValue){
                  $colors[] = [$value/$maxValue, $serieColors[$i % count($serieColors)]];
                  $i++;
              } else {
                $colors[] = [$value/$maxValue, $serieColors[$i % count($serieColors)]];
                $i++;
              }
          }
        } else {
          $i = 0;
          foreach($this->sortedRangeValues as $value){
              if($value != $minValue){
                  $colors[] = [$value/$maxValue, $this->random_color()];
                  $i++;
              }
          }
        }
        return $colors;
    }

    protected function random_color($start = 0x000000, $end = 0xFFFFFF) {
      return sprintf('#%06x', mt_rand($start, $end));
    }

    protected function getDataLabels(){
        $widget = $this->widget;
        $showDataLabel = true;
        $showAsPercent = false;
        if(!empty($widget["serieShowValueLabels"])){
            $showDataLabel = $widget["serieShowValueLabels"] != "false";
            $showAsPercent = $widget["serieShowValueLabels"] == "%";
        }
        return [
            "enabled" => $showDataLabel,
            "position" => "top",
            "textAnchor" => "middle",
            "showAsPercent" => $showAsPercent,
        ];
    }

    protected function getTitle(){
        $widget = $this->widget;

        return [
            "text" => $widget['name'],
            "align" => "left",
            "textStyle" => [
                "fontSize" => "16px",
                "fontFamily" => "Arial Black",
                "fontWeight" => "bold",
            ]
        ];
    }

    protected function getNoData(){
        $widget = $this->widget;

        return [
            "text" => !empty($widget["chartMessageNoData"]) ? $widget["chartMessageNoData"] : self::NO_DATA_TEXT,
            "align" => 'center',
            "verticalAlign" => 'middle',
            "offsetX" => 0,
            "offsetY" => 0,
            "style" => [
              "fontSize" => '14px',
            ],
        ];
    }

    protected function getOptions($widget){
      if (!empty($widget)) {
        $this->sortRangeValues();
        return [
            "title" => $this->getTitle(),
            'series' => $this->getSeries(),
        ];
      }
      return [
        "title" => $this->getTitle(),
      ];
    }

    public function convertToNext():array {
        $this->fetchWidgetData();
        $widget = $this->widget;
        $widget["gaugeData"] = empty($this->widgetData) ? [] : $this->widgetData[0];
        $mask = null;
        if (array_key_exists('serieValueFormat', $widget) === true) {
            $mask = $widget["serieValueFormat"];
        }
        if (!empty($widget["gaugeData"])) {
        $indicatorNext = [
            "name" => "apexChart_" . $widget['id'],
            "id" => $widget['id'],
            "widget" => $widget,
            "viewTypeId" => self::TYPE_ID_GAUGE,
            "component" => "ZdECharts",
            "height"    => "100%",
            "fillHeight" => true,
            "cssStyle" => self::CHART_CSS_STYLE,
            "options" => $this->getOptions($widget["gaugeData"]),
            "mask" => $mask,
        ];
          $this->widget = $indicatorNext;
          return $indicatorNext;
        }
        $chartMessageNoData = 'Não há dados para o período selecionado!';
        if (array_key_exists('chartMessageNoData', $widget) == true) $chartMessageNoData = $widget['chartMessageNoData'];
        return [
            "name"=> "textResult",
            "component"=> "ZdText",
            "text"=> $chartMessageNoData,
            "tag"=> "h1"
        ];
    }
}
