<?php

namespace Service\ChartView;

use Zeedhi\Framework\DependencyInjection\InstanceManager;
class Line extends ChartViewAbstract implements ChartViewInterface {

    const TYPE_ID_LINE = "585acc747729e554c8693172";

    public $widget;
    public $widgetData;

    function __construct($widget = null) {
        $this->widget = $widget;
        $this->widgetData = [];
    }

    protected function getXAxis(){
        $widget = $this->widget;
        $data = $this->widgetData;
        $categoria = [];
        foreach ($data as $row) {
            $categoria[] = $row[$widget['chartXAxisField']];
        }
        $xAxis = [
            "mask" => !empty($widget['chartXAxisFormat']) ? $widget['chartXAxisFormat'] : null,
            "categories" => $categoria,
            "field" => $widget['chartXAxisField'],
            "labels" => [
                "show" => true,
                "rotate" => !empty($widget['chartXAxisTicksRotation']) ? $widget['chartXAxisTicksRotation'] : -60,
            ],
            "tickPlacement" => "between"
        ];

        if($xAxis["labels"]["rotate"] != 0){
            $xAxis["labels"]["rotateAlways"] = false;
        }

        return $xAxis;
    }

    protected function getYAxis(){
        $widget = $this->widget;
        $yAxis = [
            "mask" => $widget['chartSeries'][0]['serieYAxisFormat'],
            "dataLabels" => [],
            "labels" => [
                "show" => true,
            ]
        ];

        if(!empty($widget['serieYAxisLabel'])){
            $yAxis['title'] = [
                "text" => $widget['serieYAxisLabel'],
                "style" => [
                    "fontSize" => "16px"
                ]
            ];
        }

        return $yAxis;
    }

    protected function getSeries(){
        $widget = $this->widget;
        $series = [];
        foreach ($this->widgetData as $row) {
            foreach ($widget['serieFields'] as $index => $fact) {
                if(empty($series[$index])){
                    $series[$index] = array(
                        "name" => $widget['serieLabels'][$index] ,
                        "field" => $fact,
                        "data" => array()
                    );
                }
                $series[$index]["data"][] = !empty($row[$fact]) ? $row[$fact] : 0;
            }
        }
        return $series;
    }

    protected function getTooltip(){
        $widget = $this->widget;
        return [
            "enabled" => !empty($widget["chartShowTooltip"]) ? $widget["chartShowTooltip"] : false
        ];
    }

    protected function getPlotOptions(){
        $barOptions = [
            "horizontal" => false,
            "dataLabels" => [
                "textAnchor" => "end",
            ],
        ];

        return [
            "bar" => $barOptions
        ];
    }

    protected function getSerieStacked(){
        $widget = $this->widget;

        if(!empty($widget['serieStacked'])){
            return $widget['serieStacked'];
        }
        return false;
    }

    protected function getChart(){
        $widget = $this->widget;

        return [
            "type" => "line",
            "id" => $widget['id'],
            "stacked" => $this->getSerieStacked(),
            "toolbar" => [
                "show" => true,
                "tools" => [
                    "download" => true,
                    "selection" => false,
                    "zoom" => false,
                    "zoomin" => false,
                    "zoomout" => false,
                    "pan" => false,
                    "reset" => false,
                    "customIcons" => []
                ],
            ]
        ];
    }

    protected function getSerieColors(){
        $widget = $this->widget;
        if(!empty($widget["chartSeries"][0]["serieColors"])){
            return $widget["chartSeries"][0]["serieColors"];
        }
        return self::DEFAULT_COLORS;
    }

    protected function getDataLabels(){
        $widget = $this->widget;
        return [
            "enabled" => !empty($widget["serieShowValueLabels"]) ? $widget["serieShowValueLabels"] : false,
            "position" => "top",
            "textAnchor" => "start",
            "offsetY" => -8,
            "style" => [
                "colors" => $this->getSerieStacked() ? [] : $this->getSerieColors()
            ],
        ];
    }

    protected function getTitle(){
        $widget = $this->widget;

        return [
            "text" => $widget['name'],
            "style" => [
                "fontSize" => "16px"
            ]
        ];
    }

    protected function getNoData(){
        $widget = $this->widget;

        return [
            "text" => !empty($widget["chartMessageNoData"]) ? $widget["chartMessageNoData"] : self::NO_DATA_TEXT,
            "align" => 'center',
            "verticalAlign" => 'middle',
            "offsetX" => 0,
            "offsetY" => 0,
            "style" => [
              "fontSize" => '14px',
            ],
        ];
    }

    protected function getLineMarkerShape(){
        $widget = $this->widget;
        if(empty($widget["serieLineMarkType"])){
            return "circle";
        }
        switch($widget["serieLineMarkType"]){
            default: case "circle": case "cross":
                return "circle";
            case "square": case "diamond": case "triangle-up": case "triangle-down":
                return "square";
        }
    }

    protected function getOptions(){
        $widget = $this->widget;

        $legendPosition = "top";
        if($widget["chartLegendPosition"]){
            $legendPosition = explode(" ",$widget["chartLegendPosition"])[0];
        }

        return [
            "colors" => $this->getSerieColors(),
            "drillDownView" => empty($widget['drillDownView']) ? null : $widget['drillDownView'],
            "plotOptions" => $this->getPlotOptions(),
            "dataLabels" => $this->getDataLabels(),
            "chart" => $this->getChart(),
            "legend" => [
                "position" => $legendPosition,
                "offsetY" => -10
            ],
            "zoom" > [
                "enabled" => true
            ],
            "markers" => [
                "size" => 4,
                "shape" => $this->getLineMarkerShape(),
            ],
            "title" => $this->getTitle(),
            "noData" => $this->getNoData(),
            "tooltip" => $this->getTooltip(),
            "xaxis" => $this->getXAxis(),
            "yaxis" => $this->getYAxis(),
            "stroke" => $this->getStroke(),
        ];
    }

    protected function getStroke(){
        $widget = $this->widget;
        $stroke = [
            "show" => true,
            "curve" => $this->getStrokeCurve(),
        ];
        if(!empty($widget["serieLineWidth"])){
            $stroke['width'] = $widget["serieLineWidth"];
        }
        return $stroke;
    }

    protected function getStrokeCurve(){
        $widget = $this->widget;
        if(empty($widget["serieLineType"])){
            return "straight";
        }
        switch($widget["serieLineType"]){
            default: case "linear":
                return "straight";
            case "basis": case "dundle": case "cardinal": case "monotone":
                return "smooth";
            case "step-before": case "step-after":
                return "stepline";
        }
    }

    public function convertToNext():array {
        $this->fetchWidgetData();
        $widget = $this->widget;

        $indicatorNext = [
            "name" => "apexChart_" . $widget['id'],
            "id" => $widget['id'],
            "widget" => $widget,
            "viewTypeId" => self::TYPE_ID_LINE,
            "component" => self::CHART_COMPONENT,
            "chartType" => $widget['serieType'],
            "height" =>"100%",
            "fillHeight" => true,
            "cssStyle" => self::CHART_CSS_STYLE,
            "options" => $this->getOptions(),
            "series" => $this->getSeries(),
        ];

        $this->widget = $indicatorNext;
        return $indicatorNext;
    }
}
