/*global angular reportMetadata  reportLogo pageWidth pageFooterTop reportData jsPDF */
var reportApp = angular.module('reportApp', []);
var COUNT = 'COUNT',
SUM = 'SUM',
MAX = 'MAX',
MIN = 'MIN',
AVG = 'AVG';
var PORTRAIT = "portrait",
LANDSCAPE = "landscape";
var FORMAT_A4 = 'a4',
FORMAT_LETTER = 'letter';
var DEFAULT_LANGUAGE = 'pt_br';

document.onselectstart = function() {
    return false;
};

function toLine(size) {
    return new Array((size || 10) + 1).join('_');
}

function resolvePropertyName(expression) {
    var regex = new RegExp("SUM.|[)]", "g");
    var property = expression.replace(regex, "");
    return property;
}

Number.prototype.i18n = {
    'en_us': {
        symbol: 'US$',
        thousands: ',',
        decimal: '.',
        precision: 2
    },
    'pt': {
        symbol: '€',
        thousands: ',',
        decimal: '.',
        precision: 2
    },
    'pt_br': {
        symbol: 'R$',
        thousands: '.',
        decimal: ',',
        precision: 2
    },
    'es': {
        symbol: '$',
        thousands: '.',
        decimal: ',',
        precision: 2
    },
    'es_co': {
        symbol: 'CO$',
        thousands: '.',
        decimal: ',',
        precision: 2
    }
};

Number.prototype.roundDec = function(c) {
    c = isNaN(c = Math.abs(c)) ? 2 : c;
    var n = this,
    m = Math.pow(10, c);
    return Math.round((+n || 0) * m) / m;
};

Number.prototype.formatFloat = function(c, d, t, lang) {
    d = d || this.i18n[lang].decimal;
    t = t || this.i18n[lang].thousands;
    var n = this;
    c = isNaN(c = Math.abs(c)) ? this.i18n[lang].precision : c;
    var s = n < 0 ? "-" : "",
    m = Math.pow(10, c),
    i = parseInt(n = Math.abs(n.roundDec(c)).toString()) + "",
    j = (j = i.length) > 3 ? j % 3 : 0;
    return s + (j ? i.substr(0, j) + t : "") + i.substr(j).replace(/(\d{3})(?=\d)/g, "$1" + t) + (c ? d + Math.round(Math.abs(n - i + 1) * m).toString().slice(1) : "");
};

Number.prototype.formatMoney = function(lang, options) {
    var currencySymbol = options.symbol || this.i18n[lang].symbol;
    return currencySymbol + ' ' + this.formatFloat(options.decimals, options.decimal, options.thousands, lang);
};

Number.prototype.formatHour = function() {
    var val = this;
    if (isNaN(val))
        return "00:00";

    var negativeSignFlag = "";

    if (val < 0) {
        val *= -1;
        negativeSignFlag = "-";
    }

    var hour = Math.floor(val);
    var fractionatedM = (val - hour) * 60;
    var minute = Math.floor(fractionatedM);
    var second = Math.round((fractionatedM - minute) * 60);

    //round the seconds, because the return is composed by hours and minutes
    if (second >= 30) {
        minute++;
        second -= 30;
    }

    if (minute >= 60) {
        hour++;
        minute -= 60;
    }

    hour = hour < 10 ? "0" + hour.toString() : hour.toString();
    minute = minute < 10 ? "0" + minute.toString() : minute.toString();
    //adds the flag, if necessary
    hour = negativeSignFlag + hour;

    return hour + ":" + minute;
};

reportApp.controller('MainController', ['$scope', '$timeout', function($scope, $timeout) {
    var PDFdoc = null,
        reportDate = new Date().toString('dd/MM/yyyy'),
        reportTime = new Date().toString('HH:mm'),
        reportVersion = reportMetadata.version,
        reportVersionData = reportMetadata.versionData,
        words = reportMetadata.words || {},
        lang = reportMetadata.lang,
        reportSummary = {},
        columns = reportMetadata.columns,
        reportGroups = reportMetadata.groups,
        reportFilter = reportMetadata.filter,
        clientLogo = reportLogo.client,
        enterpriseLogo = reportLogo.enterprise,
        productLogo = reportLogo.product,
        pageFormat = FORMAT_A4,
        currentPageNumber = 0,
        currentYPosition = 0,
        pageMargin = 10, // mm
        detailMargin = 0,
        images = {},
        pageWidth = -1,
        pageHeight = -1,
        pageFooterHeight = 10,
        pageFooterTop = 0,
        i18n = function(message) {
            return words[message] || message;
        },
        columnGap = 10;

    if (!lang) {
        lang = DEFAULT_LANGUAGE;
    }

    $scope.widgetLabel = reportMetadata.widgetLabel;
    $scope.orientation = reportMetadata.orientation || PORTRAIT;
    $scope.reportTitle = reportMetadata.title;
    $scope.reportData = reportData || [];

    window.addEventListener("message", function(event) {
        var data = JSON.parse(event.data);
        if (data.reportData) {
            $scope.reportData = $scope.reportData.concat(data.reportData);
            window.reportData = window.reportData.concat(data.reportData);
        }
        if (data.buildReport)
            $scope.buildReport();

    }, false);

    function normalizeValue(value, format) {
        if (typeof value === "string" && isNaN(value) && format) {
            var langOptions = Number.prototype.i18n[lang];
            var inputThousands = format.inputThousands || (format.params && format.params.thousands) || langOptions.thousands;
            var inputDecimal = format.inputDecimal || (format.params && format.params.decimal) || langOptions.decimal;

            if (inputThousands) {
                value = value.replace(new RegExp("\\" + inputThousands, 'g'), '');
            }

            if (inputDecimal) {
                value = value.replace(new RegExp("\\" + inputDecimal, 'g'), '.');
            }
        }

        return value;
    }

    function evaluateProperty(data, property) {
        var valueProperty = 0;
        if (data[property] !== null) {
            var value = normalizeValue(data[property], reportMetadata.columns[property].format);
            valueProperty = parseFloat(value);
        }
        return valueProperty;
    }

    function resetVariables() {
        if (reportSummary !== undefined) {
            reportSummary = {};
        }
        images = {};
    }

    function setFontStyle(fontSize, fontStyle) {
        PDFdoc.setFontSize(fontSize);
        PDFdoc.setFontType(fontStyle);
    }

    function loadImage(url, alias) {
        return new Promise(function(resolve, reject) {
            var img = new Image();
            img.onerror = function() {
                console.warn('cannot load image ' + url);
                resolve();
            };
            img.onload = function() {
                images[alias] = {
                    'imageData': img,
                    'cached': false
                };
                resolve();
            };
            img.src = url;
        });
    }

    function loadImages(callback) {
        var images = [
            loadImage(clientLogo, 'client'),
            loadImage(enterpriseLogo, 'enterprise'),
            loadImage(productLogo, 'product')
        ];

        return Promise.all(images);
    }

    function addImage(alias, x, y, w, h) {
        if (images[alias] !== undefined) {
            if (images[alias].cached) {
                PDFdoc.addImage(alias, x, y, w, h);
            } else {
                if (images[alias].imageData.src) {
                    var format = images[alias].imageData.src.toLowerCase().indexOf('.png') ? 'PNG' : 'JPEG';
                    var imageWidth = images[alias].imageData.width;
                    var imageHeight = images[alias].imageData.height;
                    if (imageWidth / imageHeight > w / h) {
                        y += (h - (imageHeight * w / imageWidth)) / 2;
                        h = 0;
                    } else {
                        x += (w - (imageWidth * h / imageHeight)) / 2;
                        w = 0;
                    }

                    PDFdoc.addImage(images[alias].imageData, format, x, y, w, h, alias);
                    images[alias].cached = true;
                } else {
                    console.warn('Cannot load image (' + alias + '). Invalid format!');
                }
            }
        }
    }

    function printLine(yPos) {
        if (yPos) {
            PDFdoc.line(pageMargin, yPos, PDFdoc.internal.pageSize.width - pageMargin, yPos);
        } else {
            currentYPosition -= 3;
            PDFdoc.line(pageMargin, currentYPosition, PDFdoc.internal.pageSize.width - pageMargin, currentYPosition);
            currentYPosition += 4;
        }
    }

    function printRect(x, y, w, h) {
        PDFdoc.rect(x, y, w, h, "F");
    }

    function getTextWidth(text) {
        return PDFdoc.getStringUnitWidth(text) * PDFdoc.internal.getFontSize() / PDFdoc.internal.scaleFactor;
    }

    function escapeValue(value, maxWidth, format) {
        var retValue = '';

        if (value) {
            retValue = value;
        }

        if (format && format.type !== "fix") {
            if (typeof retValue === "string" && isNaN(retValue)) {
                var langOptions = Number.prototype.i18n[lang];
                var inputThousands = format.params.inputThousands || format.params.thousands || langOptions.thousands;
                var inputDecimal = format.params.inputDecimal || format.params.decimal || langOptions.decimal;

                if (inputThousands) {
                    retValue = retValue.replace(new RegExp("\\" + inputThousands, 'g'), '');
                }

                if (inputDecimal) {
                    retValue = retValue.replace(new RegExp("\\" + inputDecimal, 'g'), '.');
                }
            }

            switch (format.type) {
                case "currency":
                    retValue = parseFloat(retValue).formatMoney(lang, format);
                    break;
                case "float":
                    retValue = parseFloat(retValue).formatFloat(format.params.precision, format.params.decimal, format.params.thousands, lang);
                    break;
                case "hour":
                    retValue = parseFloat(retValue).formatHour();
                    break;
                case "percent":
                    retValue = parseFloat(retValue).formatFloat(format.params.precision, format.params.decimal, format.params.thousands, lang)+'%';
                    break;
                case "line":
                    retValue = toLine(format.params.size);
                    break;
            }
        }

        retValue = retValue.toString();

        if (maxWidth) {
            while (retValue.length > 0 && getTextWidth(retValue) > maxWidth) {
                retValue = retValue.substring(0, retValue.length - 1);
            }
        }

        return retValue;
    }

    function printText(x, y, text, align) {
        switch (align) {
            case 'center':
                x -= getTextWidth(text) / 2;
                break;
            case 'right':
                x -= getTextWidth(text);
                break;
        }
        PDFdoc.text(x, y, text);
    }

    function initPDF() {
        PDFdoc = new jsPDF($scope.orientation, 'mm', pageFormat);
        PDFdoc.setProperties({
            title: $scope.reportTitle,
            author: 'Zeedhi',
            creator: 'ZEEDHI'
        });
        pageWidth = PDFdoc.internal.pageSize.width - (pageMargin * 2);
        pageHeight = PDFdoc.internal.pageSize.height - (pageMargin * 2);
        pageFooterTop = pageMargin + pageHeight - pageFooterHeight;
        if (reportGroups) {
            detailMargin = 5 * reportGroups.length;
        }
        initGroups();

        // PDFdoc.addFont('OpenSans', 'opensans', 'normal');
        // PDFdoc.setFont('opensans');

        setFontStyle(8, "normal");
        PDFdoc.setLineWidth(0.1);
    }

    function initGroups() {
        Object.keys(columns).sort(function(a, b) {
            return (columns[a].sequence || 0) - (columns[b].sequence || 0);
        }).forEach(function(key) {
            if (columns[key].expression) {
                reportGroups.forEach(function(group) {
                    group.aggregation = group.aggregation || {};
                    group.aggregation[key] = {
                        sum: 0,
                        count: 0
                    };
                });

                reportSummary[key] = {
                    sum: 0,
                    count: 0
                };
            }
        });
    }

    $scope.buildReport = function() {
        resetVariables();
        document.title = $scope.reportTitle;

        initPDF();

        loadImages().then(function() {
            newPage(false);
            angular.forEach($scope.reportData, function(item) {
                printDetail(item);
            });

            printGroupFooters();
            printReportSummary();

            var PDFOutput = PDFdoc.output('datauristring');
            $scope.viewer.attr('src', PDFOutput);
        });
    };

    function newPage(addPage) {
        if (addPage !== false) {
            PDFdoc.addPage();
        }
        currentPageNumber = PDFdoc.internal.getNumberOfPages();
        currentYPosition = pageMargin;
        printPageHeader();
        printPageFooter();
        printGroupHeaders();
    }

    function printPageHeader() {
        var pageHeaderHeight = 15;

        addImage('client', pageMargin, currentYPosition, 30, 10);

        setFontStyle(12, "bold");
        printText(PDFdoc.internal.pageSize.width / 2, currentYPosition + 7, $scope.reportTitle, 'center');
        setFontStyle(8, "normal");
        printText(PDFdoc.internal.pageSize.width - pageMargin, currentYPosition + 5, i18n('Página') + ': ' + currentPageNumber, 'right');
        printText(PDFdoc.internal.pageSize.width - pageMargin, currentYPosition + 8.5, reportDate + ' ' + reportTime, 'right');

        currentYPosition += pageHeaderHeight;

        printFilterHeader();
        printColumnsHeader();
    }

    function checkFilterHorizontalOverflow(currentXPosition, value){
        return currentXPosition + getTextWidth(escapeValue(value)) > PDFdoc.internal.pageSize.width - (pageMargin * 2);
    }

    function printFilterHeader() {
        var filterLineHeight = 4;
        if (reportFilter && reportFilter.length) {
            var filterHeight = (reportFilter.length * filterLineHeight) + 2;

            printLine();

            reportFilter.forEach(function(item) {
                var labelText = escapeValue((item.label || '').toUpperCase() + ": ");
                var currentXPosition = pageMargin;

                PDFdoc.setFillColor(220, 220, 220);
                printRect(currentXPosition, currentYPosition - 3.7, PDFdoc.internal.pageSize.width - (pageMargin * 2), filterLineHeight + 0.3);    
                setFontStyle(8, "bold");                
                printText(currentXPosition, currentYPosition, labelText);
                setFontStyle(8, "normal");                
                currentXPosition += getTextWidth(labelText) + 1;

                var value = item.value.split(" ");
                value.forEach(function(text){
                    if(checkFilterHorizontalOverflow(currentXPosition, text)){
                        currentXPosition = pageMargin;
                        currentYPosition += filterLineHeight;

                        PDFdoc.setFillColor(220, 220, 220);
                        printRect(currentXPosition, currentYPosition - 3.7, PDFdoc.internal.pageSize.width - (pageMargin * 2), filterLineHeight + 0.3);                               
                        printText(currentXPosition,currentYPosition, text + " ");
                        currentXPosition += getTextWidth(text + " ");                        
                    }
                    else {
                        printText(currentXPosition,currentYPosition, text);                        
                        currentXPosition += getTextWidth(text + " ");
                    }
                });
                currentYPosition += filterLineHeight;        
            });
        }
    }

    function printColumnsHeader() {
        var columnsHeaderHeight = 5;
        printLine();

        setFontStyle(8, "bold");
        var x = pageMargin + detailMargin;
        var orderedColumnsKeys = Object.keys(columns).sort(function(a, b) {
            return (columns[a].sequence || 0) - (columns[b].sequence || 0);
        });
        angular.forEach(orderedColumnsKeys, function(key) {
            var value = columns[key];
            if (typeof value.size === "string" && value.size.indexOf('%') !== -1) {
                value.columnWidth = parseInt(value.size) * (pageWidth - detailMargin) / 100;
            } else {
                value.columnWidth = parseInt(value.size) / (PDFdoc.internal.scaleFactor * 1.5);
            }
            value.columnWidthPrint = value.columnWidth - (columnGap / (PDFdoc.internal.scaleFactor * 1.5));
            switch (value.align) {
                case 'center':
                    value.alignDelta = value.columnWidthPrint / 2;
                    break;
                case 'right':
                    value.alignDelta = value.columnWidthPrint;
                    break;
                default:
                    value.align = null;
                    value.alignDelta = 0;
            }
            value.xPos = x + value.alignDelta;
            var description = value.description || value.descricao;
            printText(value.xPos, currentYPosition, escapeValue((description || '').toUpperCase(), value.columnWidthPrint), value.align);
            x += value.columnWidth;
        });
        currentYPosition += columnsHeaderHeight;
        printLine();
    }

    function printPageFooter() {
        printLine(pageFooterTop);

        setFontStyle(6, "normal");
        printText(pageMargin, pageFooterTop + 4, i18n('Relatório') + ":  " + escapeValue($scope.reportTitle));
        printText(pageMargin, pageFooterTop + 7, i18n('Versão') + ":  " + escapeValue(reportVersion));

        if (reportVersionData && reportVersionData.length > 0) {
            var top = 4;
            reportVersionData.forEach(function(data) {
                printText(PDFdoc.internal.pageSize.width - pageMargin, pageFooterTop + top, data.name + ":  " + data.version, 'right');
                top += 3;
            });
        }

        addImage('enterprise', pageMargin + 65, pageFooterTop + 2, 13, 5);
        addImage('product', pageMargin + pageWidth - 80, pageFooterTop + 2, 15, 5);
    }

    function printDetail(data) {
        var detailHeight = 4,
            firstColumn = true,
            columnIndent = 0;

        printGroupHeaders(data);

        checkPageOverflow(detailHeight);

        angular.forEach(columns, function(column, key) {
            setFontStyle(8, "normal");
            columnIndent = 0;
            if (reportMetadata.hasRowLevel) {
                if (data._hasChildrenReportMetadata) {
                    setFontStyle(8, "bold");
                }

                if (firstColumn) {
                    columnIndent = parseInt(data._rowLevelReportMetadata) * 5;
                }
            }

            printText(column.xPos + columnIndent, currentYPosition, escapeValue(data[key], column.columnWidthPrint, column.format), column.align);
            firstColumn = false;
        });
        currentYPosition += detailHeight;

        calcSummaryExpression(data);
    }

    function checkPageOverflow(height) {
        var overflow = currentYPosition + height > pageFooterTop;
        if (overflow) {
            newPage();
        }
        return overflow;
    }

    function printGroupHeaders(data) {
        var groupHeaderHeight = 4;

        function printGroupHeader(group, index) {
            if(!checkPageOverflow(groupHeaderHeight)){
                setFontStyle(8, "bold");
                printText(pageMargin + (index * 5), currentYPosition, escapeValue(group.header.label) + ': ' + escapeValue(group.lastData[group.header.displayfield]));
                currentYPosition += groupHeaderHeight;
            }
        }

        for (var i = 0; i < reportGroups.length; i++) {
            var group = reportGroups[i];
            if (data !== undefined) {
                if (group.lastData === undefined || group.lastData[group.field] != data[group.field]) {
                    if (group.lastData !== undefined) {
                        for (var j = reportGroups.length - 1; j >= i; j--) {
                            printGroupFooters(reportGroups[j], i);
                        }
                        if (group.newPage) {
                            newPage();
                        }
                    }

                    group.lastData = data;
                    resetFooterAggregation(group);

                    printGroupHeader(group, i);
                }

                addFooterAggregation(group, data);
            } else {
                if (group.lastData !== undefined) {
                    printGroupHeader(group, i);
                }
            }
        }
    }

    function resetFooterAggregation(group) {
        if (group.aggregation) {
            Object.keys(group.aggregation).forEach(function(key) {
                group.aggregation[key].count = 0;
                group.aggregation[key].sum = 0;
                group.aggregation[key].min = Math.min();
                group.aggregation[key].max = Math.max();
            });
        }
    }

    function addFooterAggregation(group, data) {
        if (group.aggregation) {
            Object.keys(group.aggregation).forEach(function(key) {
                if (typeof data[key] != 'undefined') {
                    var valueProperty = evaluateProperty(data, key);
                    if (columns[key].expression == COUNT || columns[key].expression == AVG) {
                        group.aggregation[key].count++;
                    }
                    if (columns[key].expression == SUM || columns[key].expression == AVG) {
                        if (columns[key].format && (columns[key].format.type == "currency" || columns[key].format.type == "float")) {
                            valueProperty = valueProperty.roundDec(columns[key].format.params.decimals);
                        }
                        group.aggregation[key].sum += valueProperty;
                    }
                    if(columns[key].expression == MIN || columns[key].expression == MAX){
                        group.aggregation[key].min = group.aggregation[key].min > valueProperty ? valueProperty : group.aggregation[key].min;
                        group.aggregation[key].max = group.aggregation[key].max < valueProperty ? valueProperty : group.aggregation[key].max;
                    }                    
                }
            });
        }
    }

    function printGroupFooters(group, index) {
        var groupFooterHeight = 4;
        var footerWithoutAggregationHeight = 2;

        index = index || 0;

        var groups = group !== undefined ? [group] : reportGroups;
        for (var i = groups.length - 1; i >= 0; i--) {
            group = groups[i];
            if (group.aggregation) {
                checkPageOverflow(groupFooterHeight * 2);

                setFontStyle(8, "bold");
                printText(pageMargin + ((index + i) * 5), currentYPosition, 'Total - ' + escapeValue(group.header.label) + ': ' + escapeValue(group.lastData[group.header.displayfield]), 'left');
                currentYPosition += groupFooterHeight;

                angular.forEach(columns, function(column, key) {
                    var groupData;

                    if (group.aggregation[key]) {
                        if (column.expression == SUM) {
                            groupData = group.aggregation[key].sum;
                        } else if (column.expression == COUNT) {
                            groupData = group.aggregation[key].count;
                        } else if (column.expression == AVG) {
                            groupData = group.aggregation[key].sum / group.aggregation[key].count;
                        } else if (column.expression == MIN) {
                            groupData = group.aggregation[key].min;
                        } else if (column.expression == MAX) {
                            groupData = group.aggregation[key].max;
                        }

                        printText(column.xPos, currentYPosition, escapeValue(groupData, column.columnWidthPrint, column.format), column.align);
                    }
                });
                currentYPosition += groupFooterHeight;

                printLine();
            }
            group.lastData = undefined;
        }
    }

    function calcSummaryExpression(data) {
        Object.keys(reportSummary).forEach(function(key) {
            if (typeof data[key] != 'undefined') {
                if (columns[key].expression == COUNT || columns[key].expression == AVG) {
                    reportSummary[key].count++;
                }
                if (columns[key].expression == SUM || columns[key].expression == AVG || columns[key].expression == MIN || columns[key].expression == MAX) {
                    var valueProperty = evaluateProperty(data, key);
                    if (columns[key].format && (columns[key].format.type == "currency" || columns[key].format.type == "float")) {
                        valueProperty = valueProperty.roundDec(columns[key].format.params.decimals);
                    }
                    reportSummary[key].sum += valueProperty;
                    reportSummary[key].min = (reportSummary[key].min == null || reportSummary[key].min > valueProperty) ? valueProperty : reportSummary[key].min;
                    reportSummary[key].max = (reportSummary[key].max == null || reportSummary[key].max < valueProperty) ? valueProperty : reportSummary[key].max;
                }
            }
        });
    }

    function printReportSummary() {
        var summaryHeight = 5;

        if (Object.keys(reportSummary).length > 0) {
            checkPageOverflow(summaryHeight * 2);

            setFontStyle(9, "bold");
            printText(pageMargin, currentYPosition, 'TOTAL', 'left');
            currentYPosition += summaryHeight;

            angular.forEach(columns, function(column, key) {
                var summaryData;

                if (reportSummary[key]) {
                    if (column.expression == SUM) {
                        summaryData = reportSummary[key].sum;
                    } else if (column.expression == COUNT) {
                        summaryData = reportSummary[key].count;
                    } else if (column.expression == AVG) {
                        summaryData = reportSummary[key].sum / reportSummary[key].count;
                    } else if (column.expression == MIN) {
                        summaryData = reportSummary[key].min;
                    } else if (column.expression == MAX) {
                        summaryData = reportSummary[key].max;
                    }

                    printText(column.xPos, currentYPosition, escapeValue(summaryData, column.columnWidthPrint, column.format), column.align);
                }
            });
            currentYPosition += summaryHeight;

            PDFdoc.setLineWidth(0.5);
            printLine();
            PDFdoc.setLineWidth(0.1);
        }
    }

    $scope.isPortrait = function() {
        return $scope.orientation == PORTRAIT;
    };

    $scope.isLandscape = function() {
        return $scope.orientation == LANDSCAPE;
    };

    function setOrientation(orientation) {
        if ($scope.orientation != orientation) {
            $scope.orientation = orientation;
            $timeout($scope.buildReport);
        }
    }

    $scope.setPortrait = function() {
        setOrientation(PORTRAIT);
    };

    $scope.setLandscape = function() {
        setOrientation(LANDSCAPE);
    };

    $scope.savePDF = function() {
        PDFdoc.save($scope.reportTitle + '.pdf');
    };
}]);

reportApp.directive('onFinishRender', ['$timeout', function($timeout) {
    return {
        restrict: 'A',
        link: function($scope, iElm, iAttrs, controller) {
            $scope.viewer = iElm;
            if (window.opener) window.opener.postMessage("READY!!!", "*");
            if ($scope.reportData.length)
                $timeout($scope.buildReport);
        }
    };
}]);